# /news_system/db_stats.py
import logging
import json
import psycopg2
import psycopg2.extras
import time, pytz
from datetime import datetime, timedelta
from .db_core import get_db_connection

logger = logging.getLogger(__name__)

def log_api_call(
    api_provider: str,
    service_model: str,
    success: bool,
    caller_component: str | None = None,
    input_tokens: int | None = None,
    output_tokens: int | None = None,
    metadata: dict | None = None
):
    """Logs an external API call attempt to the database."""
    try:
        with get_db_connection() as conn:
            cursor = conn.cursor()
            # Convert metadata dict to JSON string if provided
            metadata_json = json.dumps(metadata) if metadata else None
            cursor.execute("""
                INSERT INTO api_usage_logs
                (api_provider, service_model, caller_component, success, input_tokens, output_tokens, metadata)
                VALUES (%s, %s, %s, %s, %s, %s, %s)
            """, (
                api_provider,
                service_model,
                caller_component,
                success,
                input_tokens,
                output_tokens,
                metadata_json
            ))
        return True
    except Exception as e:
        logger.error(f"Failed to log API call ({api_provider} - {service_model}): {e}", exc_info=True)
        return False

def log_agent_run(
    agent_name: str,
    start_time: float,
    end_time: float,
    success: bool
):
    """Logs the start time, end time, duration, and success status of an agent run."""
    duration = end_time - start_time
    start_ts = datetime.fromtimestamp(start_time, tz=pytz.utc) # Make timestamp timezone-aware (UTC)
    end_ts = datetime.fromtimestamp(end_time, tz=pytz.utc)     # Make timestamp timezone-aware (UTC)

    try:
        with get_db_connection() as conn:
            cursor = conn.cursor()
            cursor.execute("""
                INSERT INTO agent_run_logs
                (agent_name, start_timestamp, end_timestamp, duration_seconds, success)
                VALUES (%s, %s, %s, %s, %s)
            """, (
                agent_name,
                start_ts,
                end_ts,
                duration,
                success
            ))
        return True
    except Exception as e:
        logger.error(f"Failed to log agent run for '{agent_name}': {e}", exc_info=True)
        return False

def get_api_usage_summary(days: int = 7) -> dict:
    """Calculates summary statistics for API usage over the last N days."""
    summary = {
        'total_calls': 0,
        'successful_calls': 0,
        'failed_calls': 0,
        'total_input_tokens': 0,
        'total_output_tokens': 0,
        'calls_by_provider': {},
        'tokens_by_model': {},
        'daily_usage': {} # { 'date': {'calls': N, 'tokens': M} }
    }
    start_date = datetime.utcnow() - timedelta(days=days)

    try:
        with get_db_connection() as conn:
            cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)

            # Overall counts and tokens
            cursor.execute("""
                SELECT
                    COUNT(*) as total_calls,
                    SUM(CASE WHEN success = TRUE THEN 1 ELSE 0 END) as successful_calls,
                    SUM(CASE WHEN success = FALSE THEN 1 ELSE 0 END) as failed_calls,
                    SUM(input_tokens) as total_input_tokens,
                    SUM(output_tokens) as total_output_tokens
                FROM api_usage_logs
                WHERE timestamp >= %s
            """, (start_date,))
            overall = cursor.fetchone()
            if overall:
                summary['total_calls'] = overall['total_calls'] or 0
                summary['successful_calls'] = overall['successful_calls'] or 0
                summary['failed_calls'] = overall['failed_calls'] or 0
                summary['total_input_tokens'] = overall['total_input_tokens'] or 0
                summary['total_output_tokens'] = overall['total_output_tokens'] or 0

            # Calls by provider
            cursor.execute("""
                SELECT api_provider, COUNT(*) as count
                FROM api_usage_logs
                WHERE timestamp >= %s
                GROUP BY api_provider
                ORDER BY count DESC
            """, (start_date,))
            summary['calls_by_provider'] = {row['api_provider']: row['count'] for row in cursor.fetchall()}

            # Tokens by model (only LLMs)
            cursor.execute("""
                SELECT service_model, SUM(input_tokens) as total_input, SUM(output_tokens) as total_output
                FROM api_usage_logs
                WHERE timestamp >= %s AND (input_tokens IS NOT NULL OR output_tokens IS NOT NULL)
                GROUP BY service_model
                ORDER BY total_input DESC NULLS LAST, total_output DESC NULLS LAST
            """, (start_date,))
            summary['tokens_by_model'] = {
                row['service_model']: {
                    'input': row['total_input'] or 0,
                    'output': row['total_output'] or 0,
                    'total': (row['total_input'] or 0) + (row['total_output'] or 0)
                } for row in cursor.fetchall()
            }

            # Daily usage (calls and total tokens)
            cursor.execute("""
                SELECT
                    DATE(timestamp) as usage_date,
                    COUNT(*) as daily_calls,
                    SUM(input_tokens + output_tokens) as daily_tokens
                FROM api_usage_logs
                WHERE timestamp >= %s
                GROUP BY usage_date
                ORDER BY usage_date ASC
            """, (start_date,))
            daily_data = cursor.fetchall()
            summary['daily_usage'] = {
                row['usage_date'].isoformat(): {
                    'calls': row['daily_calls'] or 0,
                    'tokens': row['daily_tokens'] or 0
                } for row in daily_data
            }

    except Exception as e:
        logger.error(f"Failed to get API usage summary: {e}", exc_info=True)
        # Return partially filled summary or empty defaults

    return summary

def get_pipeline_runtime_stats(days: int = 7) -> dict:
    """Calculates average pipeline runtime and trend over the last N days."""
    stats = {
        'average_runtime_seconds': None,
        'daily_average_runtime': {} # { 'date': avg_seconds }
    }
    start_date = datetime.utcnow() - timedelta(days=days)

    try:
        with get_db_connection() as conn:
            cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
            # Overall average for successful runs
            cursor.execute("""
                SELECT AVG(duration_seconds) as avg_runtime
                FROM agent_run_logs
                WHERE agent_name = 'Pipeline' AND success = TRUE AND start_timestamp >= %s
            """, (start_date,))
            overall_avg = cursor.fetchone()
            if overall_avg and overall_avg['avg_runtime'] is not None:
                stats['average_runtime_seconds'] = float(overall_avg['avg_runtime'])

            # Daily average runtime
            cursor.execute("""
                SELECT DATE(start_timestamp) as run_date, AVG(duration_seconds) as daily_avg
                FROM agent_run_logs
                WHERE agent_name = 'Pipeline' AND success = TRUE AND start_timestamp >= %s
                GROUP BY run_date
                ORDER BY run_date ASC
            """, (start_date,))
            daily_data = cursor.fetchall()
            stats['daily_average_runtime'] = {
                row['run_date'].isoformat(): float(row['daily_avg']) for row in daily_data if row['daily_avg'] is not None
            }
    except Exception as e:
        logger.error(f"Failed to get pipeline runtime stats: {e}", exc_info=True)

    return stats

def get_agent_runtime_stats(days: int = 7) -> dict:
    """Calculates average runtime per agent type over the last N days."""
    stats = {} # { 'agent_name': avg_seconds }
    start_date = datetime.utcnow() - timedelta(days=days)

    try:
        with get_db_connection() as conn:
            cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
            # Average runtime per agent (excluding pipeline itself)
            cursor.execute("""
                SELECT agent_name, AVG(duration_seconds) as avg_runtime
                FROM agent_run_logs
                WHERE agent_name != 'Pipeline' AND success = TRUE AND start_timestamp >= %s
                GROUP BY agent_name
                ORDER BY avg_runtime DESC
            """, (start_date,))
            agent_data = cursor.fetchall()
            stats = {row['agent_name']: float(row['avg_runtime']) for row in agent_data if row['avg_runtime'] is not None}
    except Exception as e:
        logger.error(f"Failed to get agent runtime stats: {e}", exc_info=True)

    return stats
